@@ -1,7 +1,7 @@
 #' Tibble constructor and validator
 #'
 #' @description
-#' \Sexpr[results=rd, stage=render]{tibble:::lifecycle("maturing")}
+#' \lifecycle{maturing}
 #'
 #' Creates or validates a subclass of a tibble.
 #' These function is mostly useful for package authors that implement subclasses
@@ -10,10 +10,10 @@
 #' `new_tibble()` creates a new object as a subclass of `tbl_df`, `tbl` and `data.frame`.
 #' This function is optimized for performance, checks are reduced to a minimum.
 #'
-#' @param x A tibble-like object
-#' @param ... Passed on to [structure()]
-#' @param nrow The number of rows, required
-#' @param class Subclasses to assign to the new object, default: none
+#' @param x A tibble-like object.
+#' @param ... Name-value pairs of additional attributes.
+#' @param nrow The number of rows, required.
+#' @param class Subclasses to assign to the new object, default: none.
 #' @param subclass Deprecated, retained for compatibility. Please use the `class` argument.
 #'
 #' @seealso
@@ -28,11 +28,12 @@
 #' # Existing row.names attributes are ignored:
 #' try(new_tibble(iris, nrow = 3))
 #'
-#' # The length of all columns must be consistent with the nrow argument:
+#' # The length of all columns must be compatible with the nrow argument:
 #' try(new_tibble(list(a = 1:3, b = 4:6), nrow = 2))
 new_tibble <- function(x, ..., nrow, class = NULL, subclass = NULL) {
   # For compatibility with tibble < 2.0.0
-  if (is.null(class)) {
+  if (is.null(class) && !is.null(subclass)) {
+    deprecate_soft("2.0.0", "tibble::new_tibble(subclass = )", "new_tibble(class = )")
     class <- subclass
   }
 
@@ -42,7 +43,7 @@ new_tibble <- function(x, ..., nrow, class = NULL, subclass = NULL) {
   x <- unclass(x)
 
   if (!is.list(x)) {
-    abort(error_new_tibble_must_be_list())
+    cnd_signal(error_new_tibble_must_be_list())
   }
 
   #' The `...` argument allows adding more attributes to the subclass.
@@ -50,20 +51,22 @@ new_tibble <- function(x, ..., nrow, class = NULL, subclass = NULL) {
 
   #' An `nrow` argument is required.
   if (missing(nrow)) {
+    cnd <- error_new_tibble_needs_nrow()
     if (length(x) >= 1) {
-      signal_soft_deprecated(error_new_tibble_needs_nrow())
-      nrow <- NROW(x[[1]])
+      deprecate_soft("2.0.0", "tibble::new_tibble(nrow = 'can\\'t be missing')",
+        details = cnd$message)
+      nrow <- vec_size(x[[1]])
     } else {
-      abort(error_new_tibble_needs_nrow())
+      cnd_signal(cnd)
     }
   }
   #' This should be an integer of length 1,
-  #' and every element of the list `x` should have [NROW()]
+  #' and every element of the list `x` should have [vctrs::vec_size()]
   #' equal to this value.
   #' (But this is not checked by the constructor).
   #' This takes the place of the "row.names" attribute in a data frame.
   if (!is_integerish(nrow, 1)) {
-    abort(error_new_tibble_needs_nrow())
+    cnd_signal(error_new_tibble_needs_nrow())
   }
 
   #' `x` must have names (or be empty),
@@ -72,7 +75,7 @@ new_tibble <- function(x, ..., nrow, class = NULL, subclass = NULL) {
     # Leaving this because creating a named list of length zero seems difficult
     names(x) <- character()
   } else if (is.null(names(x))) {
-    abort(error_names_must_be_non_null())
+    cnd_signal(error_names_must_be_non_null())
   }
 
   set_tibble_subclass(x, nrow, class)
@@ -91,51 +94,62 @@ validate_tibble <- function(x) {
   check_minimal_names(x)
 
   #' and that all columns are vectors, data frames or matrices.
-  check_valid_cols(x)
+  check_valid_cols(unclass(x))
 
   #' It also makes sure that all columns have the same length,
-  #' and that [NROW()] is consistent with the data.
-  validate_nrow(names(x), col_lengths(x), NROW(x))
-
-  #' 1d arrays are not supported.
-  map(x, check_no_dim)
+  #' and that [vctrs::vec_size()] is consistent with the data.
+  validate_nrow(names(x), col_lengths(x), vec_size(x))
 
   x
 }
 
+cnd_signal_if <- function(x) {
+  if (!is.null(x)) {
+    cnd_signal(x)
+  }
+}
+
+check_minimal <- function(name) {
+  cnd_signal_if(cnd_names_non_null(name))
+  cnd_signal_if(cnd_names_non_na(name))
+}
+
+check_minimal_names <- function(x) {
+  check_minimal(names(x))
+  invisible(x)
+}
+
 col_lengths <- function(x) {
-  map_int(x, NROW)
+  map_int(x, vec_size)
 }
 
 validate_nrow <- function(names, lengths, nrow) {
   # Validate column lengths, don't recycle
   bad_len <- which(lengths != nrow)
   if (has_length(bad_len)) {
-    abort(error_inconsistent_cols(nrow, names, lengths, "`nrow` argument"))
+    cnd_signal(error_incompatible_size(nrow, names, lengths, "Requested with `nrow` argument"))
   }
 }
 
 update_tibble_attrs <- function(x, ...) {
-  # Can't use structure() here because it breaks the row.names attribute
-  attribs <- list(...)
-  if (has_length(attribs)) {
-    attributes(x)[names(attribs)] <- attribs
-  }
-
-  x
+  .Call(`tibble_update_attrs`, x, pairlist2(...))
 }
 
 tibble_class <- c("tbl_df", "tbl", "data.frame")
 
-# Two dedicated functions for faster subsetting
-set_tibble_class <- function(x, nrow) {
+# Two dedicated functions for faster creation
+set_tibble_subclass <- function(x, nrow, subclass) {
   attr(x, "row.names") <- .set_row_names(nrow)
-  class(x) <- tibble_class
+  class(x) <- c(setdiff(subclass, tibble_class), tibble_class)
   x
 }
 
-set_tibble_subclass <- function(x, nrow, subclass) {
-  attr(x, "row.names") <- .set_row_names(nrow)
-  class(x) <- c(subclass, tibble_class)
-  x
+# Errors ------------------------------------------------------------------
+
+error_new_tibble_must_be_list <- function() {
+  tibble_error("`x` must be a list.")
+}
+
+error_new_tibble_needs_nrow <- function() {
+  tibble_error("`x` must be a scalar integer.")
 }
